/*****************************************************************************
	AmyEdit - A GTKMM LaTeX Editor
	Copyright (C) 2004-2005 Rob Page
	email:  page <dot> rob <at> gmail <dot> com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "Preferences.hh"
#include "AmyEdit.hh"
#include "Utils.hh"
#include "TextListViewComboBox.hh"

#include <gtkmm/stock.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/adjustment.h>
#include <gtkmm/separator.h>

#include <glib/gstdio.h> /* for g_unlink */

#include <aspell.h>
#include <fstream>

#include <map>

Gtk::Widget *Preferences::new_heading(Glib::ustring text)
{
	Gtk::Label* label = Gtk::manage(new Gtk::Label("", Gtk::ALIGN_LEFT));
	label->set_markup("<span weight='bold' size='larger'>" + text + "</span>");

	return (label);
}

Gtk::Widget *Preferences::new_item(Glib::ustring text, Gtk::Widget *widget, bool fill, bool ralign)
{
	Gtk::HBox *hbox = Gtk::manage(new Gtk::HBox(false, 5));
	hbox->pack_start(*Gtk::manage(new Gtk::Label("        " + text, Gtk::ALIGN_LEFT)), false, false, 0);
	if (ralign)
		hbox->pack_end(*widget, fill, fill, 0);
	else
		hbox->pack_start(*widget, fill, fill, 0);
	return (hbox);
}

Gtk::Widget *Preferences::new_button(Glib::ustring text, sigc::slot<void> fn_slot){
	Gtk::Alignment *align = Gtk::manage(new Gtk::Alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0));
	Gtk::Button *button = Gtk::manage(new Gtk::Button(text));
	button->signal_clicked().connect(fn_slot);
	align->add(*button);
	return align;
}

Preferences::Preferences()
{
	PrefStruct* myPrefs = PrefStruct::instance();
	set_transient_for(*AmyEdit::instance());

	set_title(_("AmyEdit Preferences"));

	Gtk::VBox *vbox = Gtk::manage(new Gtk::VBox(FALSE, 4));
	set_border_width(12);

	/*************************************
	 * First Preferences Group : Editing *
	 *************************************/
	
	vbox->add(*new_heading(_("Editing")));

	Gtk::Adjustment* adj = Gtk::manage(new Gtk::Adjustment(myPrefs->get<int>("General", "tab_size"),1,32));
	tabsizebutton = Gtk::manage(new Gtk::SpinButton(*adj, 1.0, 0));
	vbox->add(*new_item(_("Tab Size:"), tabsizebutton));

	highlightbutton = Gtk::manage(new Gtk::CheckButton(_("Syntax Highlighting")));
	highlightbutton->set_active(myPrefs->get<bool>("General", "syntax_highlighting"));
	vbox->add(*new_item("", highlightbutton));
	
	/***************************************
	 * Second Preferences Group : Spelling *
	 ***************************************/

	vbox->add(*new_heading(_("Spellcheck")));
	
	languageslist = Gtk::manage(new TextListViewComboBox());
	{
		using namespace Gtk::Menu_Helpers;

		AspellConfig *config;
		AspellDictInfoList *dlist;
		AspellDictInfoEnumeration * dels;
		const AspellDictInfo * entry;

		config = new_aspell_config();

		/* the returned pointer should _not_ need to be deleted */
		dlist = get_aspell_dict_info_list(config);

		/* config is no longer needed */
		delete_aspell_config(config);

		dels = aspell_dict_info_list_elements(dlist);

		while ( (entry = aspell_dict_info_enumeration_next(dels)) != 0){
			languageslist->get_model()->append(entry->name);
		}

		delete_aspell_dict_info_enumeration(dels);

	}
	languageslist->select(myPrefs->get<Glib::ustring>("General", "language"));
	vbox->add(*new_item(_("Spellcheck Language:"), languageslist, true));
	
	/***************************************
	 * Fourth Preferences Group : Commands *
	 ***************************************/
	
	vbox->add(*new_heading(_("Commands")));

	previewoptionslist = Gtk::manage(new TextListViewComboBox());	
	Amy::populate_text_list_model_from_list(previewoptionslist->get_model(), 
	                                        myPrefs->get_keys("Commands"));
	previewoptionslist->select(myPrefs->get<Glib::ustring>("General", "preview_combo"));
	vbox->add(*new_item(_("LaTeX Command:"), previewoptionslist, true));
	vbox->add(*new_button(_("Edit Commands..."), sigc::mem_fun(*this, &Preferences::edit_latex_commands)));

	/**********************************
	 * Fifth Preferences Group : Misc *
	 **********************************/

	vbox->add(*new_heading(_("Other Settings")));
	spacesinfilesbutton = Gtk::manage(new Gtk::CheckButton(_("Warn About Spaces in Filenames")));
	spacesinfilesbutton->set_active(myPrefs->get<bool>("General", "warn_about_filenames_with_spaces"));
	vbox->add(*new_item("", spacesinfilesbutton));
	
	/*************************************
	 * Setting Up The Rest of the Dialog *
	 *************************************/
	get_vbox()->add(*vbox);
	set_skip_taskbar_hint();
	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

	show_all_children();
}

Preferences::~Preferences()
{

}

int Preferences::run()
{
	int result=Gtk::Dialog::run();

	if (result==Gtk::RESPONSE_OK){
		PrefStruct* prefs = PrefStruct::instance();
		prefs->set("General", "tab_size",(int)tabsizebutton->get_value());
		prefs->set("General", "language",languageslist->get_selected());
		prefs->set("General", "syntax_highlighting",highlightbutton->get_active());
		prefs->set("General", "warn_about_filenames_with_spaces",spacesinfilesbutton->get_active());

		prefs->set("General", "preview_combo",previewoptionslist->get_selected());
	}

	return result;

}

void Preferences::edit_latex_commands(){
	EditLatexCommands dialog(this);
	dialog.run();
	Amy::populate_text_list_model_from_list(previewoptionslist->get_model(), PrefStruct::instance()->get_keys("Commands"), true);
	previewoptionslist->select(PrefStruct::instance()->get<Glib::ustring>("General", "preview_combo"));
}


Preferences::EditLatexCommands::EditLatexCommands(Preferences* parent){
	Gtk::Dialog dialog(_("Edit Commands"));
	set_transient_for(*parent);
	
	Gtk::Button* b = Gtk::manage(new Gtk::Button(Gtk::Stock::HELP));
	b->signal_clicked().connect(sigc::mem_fun(*this, &Preferences::EditLatexCommands::on_help_clicked));

	get_action_area()->add(*b);
	get_action_area()->set_child_secondary(*b);
	
	add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);

	Gtk::VBox *vbox=Gtk::manage(new Gtk::VBox(false, 4));
	get_vbox()->add(*vbox);
	set_border_width(12);

	Gtk::HBox* hb = Gtk::manage(new Gtk::HBox(false, 4));
	Gtk::Label* label = Gtk::manage(new Gtk::Label(_("<b>Current Command</b>")));
	label->set_use_markup(true);
	hb->add(*label);
		
	combobox = Gtk::manage(new TextListViewComboBox());
	Amy::populate_text_list_model_from_list(combobox->get_model(),
	              PrefStruct::instance()->get_keys("Commands"));
	last_combo = PrefStruct::instance()->get<Glib::ustring>("General", "preview_combo");
	combobox->select(last_combo);
	combobox->signal_changed().connect(sigc::mem_fun(*this, &Preferences::EditLatexCommands::on_combo_changed));
	hb->add(*combobox);	

	Gtk::IconSize iconsize(Gtk::ICON_SIZE_BUTTON);
	
	b = Gtk::manage(new Gtk::Button());
	b->add(*Gtk::manage(new Gtk::Image(Gtk::Stock::ADD, iconsize)));
	b->signal_clicked().connect(sigc::mem_fun(*this, &Preferences::EditLatexCommands::on_add_clicked));
	hb->pack_start(*b, false, false);
	
	b = Gtk::manage(new Gtk::Button());
	b->add(*Gtk::manage(new Gtk::Image(Gtk::Stock::REMOVE, iconsize)));
	b->signal_clicked().connect(sigc::mem_fun(*this, &Preferences::EditLatexCommands::on_remove_clicked));
	hb->pack_start(*b, false, false);

	vbox->add(*hb);
				
	cmd_entry = Gtk::manage(new Gtk::Entry());
	vbox->add(*Preferences::new_item(_("Latex Command"), cmd_entry, false, true));

	preview_entry = Gtk::manage(new Gtk::Entry());
	vbox->add(*Preferences::new_item(_("Preview Command"), preview_entry, false, true));

	output_entry = Gtk::manage(new Gtk::Entry());
	vbox->add(*Preferences::new_item(_("Output File"), output_entry, false, true));

	set_entrys(last_combo);
	show_all_children();

}

void Preferences::EditLatexCommands::on_add_clicked(){
	Amy::MiniDialog dialog(_("Add Command Set"), "", Amy::MISC);
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::ADD, Gtk::RESPONSE_OK);

	Gtk::HBox* hbox = Gtk::manage(new Gtk::HBox(4, false));
	hbox->add(*Gtk::manage(new Gtk::Label(_("Enter a name for this set"))));
	Gtk::Entry* entry = Gtk::manage(new Gtk::Entry());
	hbox->add(*entry);
	dialog.get_vbox()->add(*hbox);
	dialog.show_all_children();
	if (dialog.run()==Gtk::RESPONSE_OK)
	{
		PrefStruct* prefs = PrefStruct::instance();
		
		Glib::ustring new_command = entry->get_text();
		if (new_command!="" && !(prefs->has_key("Commands", new_command))){
			prefs->set("Commands", new_command, Glib::ustring(";;;"));
			Amy::populate_text_list_model_from_list(combobox->get_model(), prefs->get_keys("Commands"), true);
			combobox->select(new_command);
		}
	}
}

void Preferences::EditLatexCommands::on_remove_clicked(){
	Glib::ustring selected = combobox->get_selected();
	if (selected!="" && Amy::Message(_("Remove Command Set"), _("Are you sure you wish to remove the command set '") + selected + _("'?"), Amy::OK_CANCEL) == Gtk::RESPONSE_OK){
		PrefStruct* prefs = PrefStruct::instance();
		prefs->remove_key("Commands", selected);
		last_combo="";
		Amy::populate_text_list_model_from_list(combobox->get_model(), prefs->get_keys("Commands"), true);
	}
}

void Preferences::EditLatexCommands::update_command(Glib::ustring command){
	std::list<Glib::ustring> cmdlist;

	cmdlist.push_back(cmd_entry->get_text());
	cmdlist.push_back(preview_entry->get_text());
	cmdlist.push_back(output_entry->get_text());
	
	PrefStruct::instance()->set("Commands", command, cmdlist);
	
}

void Preferences::EditLatexCommands::on_combo_changed(){
	PrefStruct* prefs = PrefStruct::instance();
	if (last_combo!="")
		update_command(last_combo);
	prefs->set("General", "preview_combo", combobox->get_selected());
	set_entrys(combobox->get_selected());
	last_combo=combobox->get_selected();
}

void Preferences::EditLatexCommands::on_help_clicked(){
	Amy::MiniDialog dialog(_("Edit Latex Commands Help"), _("This dialog allows you to enter the commands you wish to be run when you click on preview or export.\n\n%%f will be replaced by the filename\n%%g will be replaced by the filename with the .tex extension removed (if it is present).\n%%h will be replaced by the expansion of the output file option.\n%%%% will be replaced by a single %%."), Amy::INFO);
	dialog.set_transient_for(*this);

	dialog.run();

}

void Preferences::EditLatexCommands::set_entrys(Glib::ustring command)
{
	PrefStruct* prefs = PrefStruct::instance();
	Glib::ustring key = prefs->get<Glib::ustring>("General", "preview_combo");
	if (key!="")
	{
		std::vector <Glib::ustring> cmds = prefs->get<Glib::ArrayHandle<Glib::ustring> >("Commands", key);

		cmd_entry->set_text(cmds.at(0));
		preview_entry->set_text(cmds.at(1));
		output_entry->set_text(cmds.at(2));
	}	
}

int Preferences::EditLatexCommands::run()
{
	int retVal = Gtk::Dialog::run();
	update_command(combobox->get_selected());
	return retVal;
}
